@@ -34,7 +34,7 @@ public:
   virtual void    set_y(const VectorXd& y_);
   virtual void    update_beta(const dblvec &beta_);
   virtual void    update_theta(const dblvec &theta_);
-  virtual void    update_u(const MatrixXd &u_);
+  virtual void    update_u(const MatrixXd &u_, bool append = false);
   virtual void    set_trace(int trace_);
   virtual dblpair marginal(const MarginType type,
                              const std::string& x,
@@ -92,20 +92,30 @@ inline void glmmr::Model<modeltype>::update_theta(const dblvec &theta_){
 }
 
 template<typename modeltype>
-inline void glmmr::Model<modeltype>::update_u(const MatrixXd &u_){
+inline void glmmr::Model<modeltype>::update_u(const MatrixXd &u_, bool append){
 #ifdef R_BUILD
   if(u_.rows()!=model.covariance.Q())Rcpp::stop(std::to_string(u_.rows())+" rows provided, "+std::to_string(model.covariance.Q())+" expected");
 #endif
   
-  if(u_.cols()!=re.u_.cols()){
+  int newcolsize = u_.cols();
+  int currcolsize = re.u_.cols();
+  
+  if(append){
+    re.u_.conservativeResize(NoChange,currcolsize + newcolsize);
+    re.zu_.conservativeResize(NoChange,currcolsize + newcolsize);
+    re.u_.rightCols(newcolsize) = u_;
+    optim.ll_current.resize(currcolsize + newcolsize,NoChange);
+  } else {
+    if(u_.cols()!=re.u_.cols()){
 #if defined(ENABLE_DEBUG) && defined(R_BUILD)
-    Rcpp::Rcout << "\nResize u: " << model.covariance.Q() << "x" << u_.cols();
+      Rcpp::Rcout << "\nResize u: " << model.covariance.Q() << "x" << u_.cols();
 #endif
-    int newcolsize = u_.cols();
-    re.u_.resize(NoChange,newcolsize);
-    re.zu_.resize(NoChange,newcolsize);
+      re.u_.resize(NoChange,newcolsize);
+      re.zu_.resize(NoChange,newcolsize);
+      re.u_ = u_;
+      if(newcolsize != optim.ll_current.rows()) optim.ll_current.resize(newcolsize,NoChange);
+    }
   }
-  re.u_ = u_;
   re.zu_ = model.covariance.ZLu(re.u_);
 }
 
@@ -230,10 +240,9 @@ inline dblpair glmmr::Model<modeltype>::marginal(const MarginType type,
   mcalc.instructions.push_back(Do::PushExtraData);
   mcalc.instructions.push_back(Do::Add);
   glmmr::linear_predictor_to_link(mcalc,model.family.link);
-  mcalc.data.conservativeResize(newXdata.rows(),newXdata.cols());
+  mcalc.data.conservativeResize(newXdata.rows(),NoChange);
   mcalc.data = newXdata;
-  mcalc.parameters.resize(model.linear_predictor.P());
-  mcalc.parameters = model.linear_predictor.parameters;
+  
   dblpair result;
   VectorXd delta = VectorXd::Zero(P);
   MatrixXd M(P,P);
@@ -316,11 +325,18 @@ inline dblpair glmmr::Model<modeltype>::marginal(const MarginType type,
       dblvec m_result(1+P);
       for(int i = 0; i < newXdata.rows(); i++)mcalc.data(i,xcol) = xvals.first;
 #pragma omp parallel for reduction(+:d_result) reduction(vec_dbl_plus:delta_vec) private(m_result)
-      for(int i = 0; i < N; i++){
+      for(int i = 0; i < N; i++)
+      {
         m_result = mcalc.calculate<CalcDyDx::BetaFirst>(i,0,0,zu(i));
         d_result += m_result[0];
         for(int p = 0; p < P; p++)delta_vec[p] += m_result[p+1];
-        for(int i = 0; i < newXdata.rows(); i++)mcalc.data(i,xcol) = xvals.second;
+      }
+      
+      for(int j = 0; j < newXdata.rows(); j++)mcalc.data(j,xcol) = xvals.second;
+      
+#pragma omp parallel for reduction(+:d_result) reduction(vec_dbl_plus:delta_vec) private(m_result)      
+      for(int i = 0; i < N; i++)
+      {
         m_result = mcalc.calculate<CalcDyDx::BetaFirst>(i,0,0,zu(i));
         d_result += -1.0*m_result[0];
         for(int p = 0; p < P; p++)delta_vec[p] += -1.0*m_result[p+1];
@@ -339,19 +355,23 @@ inline dblpair glmmr::Model<modeltype>::marginal(const MarginType type,
       dblvec m_result0(P+1);
       dblvec m_result1(P+1);
       for(int i = 0; i < newXdata.rows(); i++) newXdata(i,xcol) = xvals.first;
-#pragma omp parallel for private(m_result0,m_result1) reduction(+:d_result0) reduction(+:d_result1) \
-      reduction(vec_dbl_plus:delta0) reduction(vec_dbl_plus:delta1) 
-      for(int i = 0; i < N; i++){
+#pragma omp parallel for private(m_result0) reduction(+:d_result0) reduction(vec_dbl_plus:delta0)  
+      for(int i = 0; i < N; i++)
+      {
         m_result0 = mcalc.calculate<CalcDyDx::BetaFirst>(i,0,0,zu(i));
-        for(int i = 0; i < newXdata.rows(); i++) mcalc.data(i,xcol) = xvals.second;
-        m_result1 = mcalc.calculate<CalcDyDx::BetaFirst>(i,0,0,zu(i));
         d_result0 += m_result0[0];
-        d_result1 += m_result1[0];
-        for(int p = 0; p < P; p++){
-          delta0[p] += m_result0[p+1];
-          delta1[p] += m_result1[p+1];
-        }
+        for(int p = 0; p < P; p++) delta0[p] += m_result0[p+1];
       }
+      
+      for(int i = 0; i < newXdata.rows(); i++) mcalc.data(i,xcol) = xvals.second;
+#pragma omp parallel for private(m_result1) reduction(+:d_result1) reduction(vec_dbl_plus:delta1) 
+      for(int i = 0; i < N; i++)
+      {
+        m_result1 = mcalc.calculate<CalcDyDx::BetaFirst>(i,0,0,zu(i));
+        d_result1 += m_result1[0];
+        for(int p = 0; p < P; p++) delta1[p] += m_result1[p+1];
+      }
+      
       result.first = log(d_result0/N) - log(d_result1/N);
       for(int p = 0; p < P; p++)delta(p) = delta0[p]/d_result0 - delta1[p]/d_result1;
       result.second = sqrt((delta.transpose()*M*delta)(0));
@@ -370,7 +390,7 @@ inline dblpair glmmr::Model<modeltype>::marginal(const MarginType type,
         double d_result = 0;
         dblvec m_result(2+2*P);
         dblvec delta_vec(P,0.0);
-        for(int i = 0; i < newXdata.rows(); i++)newXdata(i,xcol) = xvals.first;
+        for(int i = 0; i < newXdata.rows(); i++) newXdata(i,xcol) = xvals.first;
 #pragma omp parallel for private(m_result) reduction(+:d_result) reduction(vec_dbl_plus:delta_vec) collapse(2)
         for(int i = 0; i < model.n(); i++){
           for(int j = 0; j < iter; j++){
@@ -396,8 +416,10 @@ inline dblpair glmmr::Model<modeltype>::marginal(const MarginType type,
         dblvec delta_vec(P,0.0);
         for(int i = 0; i < newXdata.rows(); i++)newXdata(i,xcol) = xvals.first;
 #pragma omp parallel for private(m_result) reduction(+:d_result) reduction(vec_dbl_plus:delta_vec) collapse(2)
-        for(int i = 0; i < model.n(); i++){
-          for(int j = 0; j < iter; j++){
+        for(int i = 0; i < model.n(); i++)
+        {
+          for(int j = 0; j < iter; j++)
+          {
             if(N==1){
               m_result = mcalc.calculate<CalcDyDx::BetaFirst>(0,0,0,re.zu_(i,j));
             } else {
@@ -405,7 +427,15 @@ inline dblpair glmmr::Model<modeltype>::marginal(const MarginType type,
             }
             d_result += m_result[0];
             for(int p = 0; p < P; p++)delta_vec[p] += m_result[p+1];
-            for(int i = 0; i < newXdata.rows(); i++)mcalc.data(i,xcol) = xvals.second;
+           }
+        }
+        
+        for(int i = 0; i < newXdata.rows(); i++)mcalc.data(i,xcol) = xvals.second;
+#pragma omp parallel for private(m_result) reduction(+:d_result) reduction(vec_dbl_plus:delta_vec) collapse(2)
+        for(int i = 0; i < model.n(); i++)
+        {
+          for(int j = 0; j < iter; j++)
+          {
             if(N==1){
               m_result = mcalc.calculate<CalcDyDx::BetaFirst>(0,0,0,re.zu_(i,j));
             } else {
@@ -413,7 +443,7 @@ inline dblpair glmmr::Model<modeltype>::marginal(const MarginType type,
             }
             d_result += -1.0*m_result[0];
             for(int p = 0; p < P; p++)delta_vec[p] += -1.0*m_result[p+1];
-           }
+          }
         }
         result.first = d_result/(N*iter);
         for(int p = 0; p < P; p++)delta(p) = delta_vec[p];
@@ -430,25 +460,34 @@ inline dblpair glmmr::Model<modeltype>::marginal(const MarginType type,
         dblvec m_result0(1+P);
         dblvec m_result1(1+P);
         for(int i = 0; i < newXdata.rows(); i++)newXdata(i,xcol) = xvals.first;
-#pragma omp parallel for private(m_result0,m_result1) reduction(+:d_result0) reduction(+:d_result1) \
-        reduction(vec_dbl_plus:delta0) reduction(vec_dbl_plus:delta1) collapse(2)
-        for(int i = 0; i < model.n(); i++){
-          for(int j = 0; j < iter; j++){
+#pragma omp parallel for private(m_result0) reduction(+:d_result0) reduction(vec_dbl_plus:delta0) collapse(2)
+        for(int i = 0; i < model.n(); i++)
+        {
+          for(int j = 0; j < iter; j++)
+          {
             if(N==1){
               m_result0 = mcalc.calculate<CalcDyDx::BetaFirst>(1,0,0,re.zu_(i,j));
-              for(int i = 0; i < newXdata.rows(); i++)mcalc.data(i,xcol) = xvals.second;
-              m_result1 = mcalc.calculate<CalcDyDx::BetaFirst>(1,0,0,re.zu_(i,j));
             } else {
               m_result0 = mcalc.calculate<CalcDyDx::BetaFirst>(i,0,0,re.zu_(i,j));
-              for(int i = 0; i < newXdata.rows(); i++)mcalc.data(i,xcol) = xvals.second;
-              m_result1 = mcalc.calculate<CalcDyDx::BetaFirst>(i,0,0,re.zu_(i,j));
             }
             d_result0 += m_result0[0];
-            d_result1 += m_result1[0];
-            for(int p = 0; p < P; p++){
-              delta0[p] += m_result0[p+1];
-              delta1[p] += m_result1[p+1];
+            for(int p = 0; p < P; p++) delta0[p] += m_result0[p+1];
+          }
+        }
+        
+        for(int i = 0; i < newXdata.rows(); i++)mcalc.data(i,xcol) = xvals.second;
+#pragma omp parallel for private(m_result1) reduction(+:d_result1) reduction(vec_dbl_plus:delta1) collapse(2)
+        for(int i = 0; i < model.n(); i++)
+        {
+          for(int j = 0; j < iter; j++)
+          {
+            if(N==1){
+              m_result1 = mcalc.calculate<CalcDyDx::BetaFirst>(1,0,0,re.zu_(i,j));
+            } else {
+              m_result1 = mcalc.calculate<CalcDyDx::BetaFirst>(i,0,0,re.zu_(i,j));
             }
+            d_result1 += m_result1[0];
+            for(int p = 0; p < P; p++) delta1[p] += m_result1[p+1];
           }
         }
         
